const ShellVersion = imports.misc.config.PACKAGE_VERSION.split(".");
const Main = imports.ui.main;
const Lang    = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const St = imports.gi.St;
const Shell   = imports.gi.Shell;
const Meta    = imports.gi.Meta;
const Gio     = imports.gi.Gio;
const Utils   = imports.misc.extensionUtils.getCurrentExtension().imports.utils; 
const mySettings = Utils.getSettings();
const key_bindings = {'toggle-terminal': function() {_toggleTerminal();}};
const WA_FOLDER = 'default-folder';
const WA_ICONPOS = 'iconpos';
const TERMINAL_SCHEMA = 'org.gnome.desktop.default-applications.terminal';
var dt = new Date();

const TerminalIconButton = new Lang.Class({
    Name: 'TerminalIcon.ApplicationsButton',
    Extends: PanelMenu.Button,
    _init: function() {
        this.parent(0.0,'TerminalIcon');
        this.actor.add_style_class_name('panel-status-button');
        this.buttonIcon = new St.Icon({icon_name: "my-terminal-symbolic", style_class: 'system-status-icon' });
        this.actor.add_actor(this.buttonIcon);
        this.actor.connect('button-press-event', Lang.bind(this, _toggleTerminal));
    }
});

function _toggleTerminal() {
	let win = _getWindowActor();
	// Start terminal + block run new terminal windows during 300ms
	if (win === 'start') {
		if (new Date().getTime() > dt.getTime() ){
			dt.setTime( new Date().getTime() + 400 );
			_startApplication();
			return;
		}				
	}

        if (!win) {
		dt.setTime( new Date().getTime() + 200 );
		return;
        }
        // minimize/unmaximize + 40ms between 2 minimize call
	if (win.has_focus()){

        	if (win.minimized) {
			if (new Date().getTime() > dt.getTime() ){
		  		win.unminimize();
		   		win.activate(global.get_current_time());
		   		dt.setTime( new Date().getTime() + 40 );
			}
       		} else {
			if (new Date().getTime() > dt.getTime() ){
				dt.setTime( new Date().getTime() + 40 );
				win.minimize();
			}
         	}

	} else {
		win.unminimize();
		win.activate(global.get_current_time());
		dt.setTime( new Date().getTime() + 200 );
	}

}



function _startApplication(){
	let folder = this.mySettings.get_string(WA_FOLDER);
        let termSettings = new Gio.Settings({ 'schema': TERMINAL_SCHEMA });
        let termExec     = termSettings.get_string('exec');
        if (termExec === 'gnome-terminal') {
           folder = ' --working-directory='+ folder ;
        }
	imports.misc.util.trySpawnCommandLine( termExec+' '+ folder);
}

function _getWindowActor(){
    let termSettings = new Gio.Settings({ 'schema': TERMINAL_SCHEMA });
    let termExec     = termSettings.get_string('exec');
	let ApplicationString = _(termExec+'.desktop');
	if (ApplicationString==_('x-terminal-emulator.desktop')){ApplicationString = _('gnome-terminal.desktop');}
	let window =Shell.AppSystem.get_default().lookup_app(ApplicationString).get_windows()[0];
	if(typeof window == 'undefined') { window = 'start';}
	return window;
}

function init(extensionMeta) {
    let theme = imports.gi.Gtk.IconTheme.get_default();
    theme.append_search_path(extensionMeta.path + "/icons");
}

let TerminalButton;

function enable() {
    TerminalButton = new TerminalIconButton();
    let iconpos = this.mySettings.get_string(WA_ICONPOS);
	let key;

    if (ShellVersion[1]>4) {
        if (iconpos === 'right') {
           Main.panel.addToStatusArea('terminalicon-menu', TerminalButton, 0, 'right');
        } else {
           Main.panel.addToStatusArea('terminalicon-menu', TerminalButton, 0, 'left');
        }
        
    } else {

        if (iconpos === 'right') {
           Main.panel._rightBox.insert_child_at_index(TerminalButton.actor,0);
        } else {
           let _children = Main.panel._leftBox.get_children();
           Main.panel._leftBox.insert_child_at_index(TerminalButton.actor,_children.length - 1);
        }
        Main.panel._menus.addMenu(TerminalButton.menu);
    }

    dt.setTime(dt.getTime());
	let key;	
	for(key in key_bindings) 
	{
		if (Main.wm.addKeybinding && Shell.KeyBindingMode) 
		{ 
			Main.wm.addKeybinding(key,mySettings,Meta.KeyBindingFlags.NONE, Shell.KeyBindingMode.NORMAL | Shell.KeyBindingMode.MESSAGE_TRAY,key_bindings[key]); 
		}
		else if (Main.wm.addKeybinding && Main.KeybindingMode) 
		{
			Main.wm.addKeybinding(key,mySettings,Meta.KeyBindingFlags.NONE,Main.KeybindingMode.NORMAL | Main.KeybindingMode.MESSAGE_TRAY, key_bindings[key]);
		} 
		else 
		{
			global.display.add_keybinding(key,mySettings,Meta.KeyBindingFlags.NONE,key_bindings[key]); 
		}
	}
}

function disable() {
	let key;
    TerminalButton.destroy();
    for(key in key_bindings) {
        global.display.remove_keybinding(key);
    }
}


