const GObject = imports.gi.GObject;
const Gdk = imports.gi.Gdk;
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const Utils = imports.misc.extensionUtils.getCurrentExtension().imports.utils; 
const mySettings = Utils.getSettings();
const WA_FOLDER = 'default-folder';
const WA_ICONPOS = 'iconpos';

// Keep enums in sync with GSettings schemas
const IconPos = {
    LEFT: 0,
    RIGHT: 1
};

// End of imports;

const pretty_names = {
	'toggle-terminal': 'Toggle terminal'
}

function init() {
}

function append_hotkey(model, settings, name, pretty_name) {
	let [key, mods] = Gtk.accelerator_parse(settings.get_strv(name)[0]);

	let row = model.insert(10);

	model.set(row, [0, 1, 2, 3], [name, pretty_name, 
		mods, key ]);
}

function changefolder(object, text) {
	this.mySettings.set_string(WA_FOLDER, text.get_text());
}

function buildPrefsWidget() {
 
	let frame = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 1 });
	let vbox1 = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin_left: 1 });
	let hboxMenu1 = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let hboxMenu2 = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});

	let label = new Gtk.Label({ label: "<b>Global</b>", use_markup: true, xalign: 0 });



	let labelMenu1 = new Gtk.Label({label: "Default folder", xalign: 0});
	let valueMenu1 = new Gtk.Entry({ hexpand: true });
	valueMenu1.set_text(this.mySettings.get_string(WA_FOLDER));
	let buttonMenu = new Gtk.Button({ label: "Apply" });
	buttonMenu.connect("clicked", Lang.bind(this, this.changefolder  , valueMenu1 ));


        // pinch
        let item = new Gtk.ComboBoxText();
        for (let type in IconPos) {
            if (IconPos.hasOwnProperty(type)) {
                let label = type[0].toUpperCase() +
                    type.substring(1).toLowerCase();
                item.insert(-1, IconPos[type].toString(), label);
            }
        }
        item.set_active_id(this.mySettings.get_enum(WA_ICONPOS).toString());
        item.connect('changed', Lang.bind(this, function (combo) {
            let value = combo.get_active_id();
            if (value !== undefined &&
                this.mySettings.get_enum(WA_ICONPOS) !== value) {
                this.mySettings.set_enum(WA_ICONPOS, value);
            }
        }));
	let labelMenu2 = new Gtk.Label({label: "Terminal icon position on gnome panel:", xalign: 0});
  

	hboxMenu1.pack_start(labelMenu1, true, true, 0);
	hboxMenu1.add(valueMenu1);
	hboxMenu1.add(buttonMenu);

	hboxMenu2.pack_start(labelMenu2, true, true, 0);
	hboxMenu2.add(item);


	vbox1.add(hboxMenu1);
	vbox1.add(hboxMenu2);
	frame.add(label);
	frame.add(vbox1);





	let model = new Gtk.ListStore();

	model.set_column_types([
		GObject.TYPE_STRING,
		GObject.TYPE_STRING,
		GObject.TYPE_INT,
		GObject.TYPE_INT
	]);

	global.log("Modal created.");

	let settings = Utils.getSettings();

	for(key in pretty_names) {
		append_hotkey(model, settings, key, pretty_names[key]);
	}

	global.log("Added hotkeys to model");

	let treeview = new Gtk.TreeView({
		'expand': true,
		'model': model
	});

	global.log("TreeView created.");

	let col;
	let cellrend;

	cellrend = new Gtk.CellRendererText();

	col = new Gtk.TreeViewColumn({
		'title': 'Keybinding',
		'expand': true
	});

	col.pack_start(cellrend, true);
	col.add_attribute(cellrend, 'text', 1);


	treeview.append_column(col);

	global.log("Column one created.");

	cellrend = new Gtk.CellRendererAccel({
		'editable': true,
		'accel-mode': Gtk.CellRendererAccelMode.GTK
	});

	cellrend.connect('accel-edited', function(rend, iter, key, mods) {
		let value = Gtk.accelerator_name(key, mods);
		
		let [succ, iter ] = model.get_iter_from_string(iter);
		
		if(!succ) {
			throw new Error("Something be broken, yo.");
		}

		let name = model.get_value(iter, 0);

		model.set(iter, [ 2, 3 ], [ mods, key ]);

		global.log("Changing value for " + name + ": " + value);

		settings.set_strv(name, [value]);
	});

	col = new Gtk.TreeViewColumn({
		'title': 'Accel'
	});

	col.pack_end(cellrend, false);
	col.add_attribute(cellrend, 'accel-mods', 2);
	col.add_attribute(cellrend, 'accel-key', 3);

	treeview.append_column(col);
	
	global.log("Column two created.");

	let win = new Gtk.ScrolledWindow({
		'vexpand': true
	});
	win.add(treeview);

	global.log("ScrolledWindow created.");
	


	global.log("Returning.");


	let hboxMenu3 = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	hboxMenu3.add(win);
	vbox1.add(hboxMenu3);


	frame.show_all();
	return frame;
}



